#import "SINClientMediator.h"
#import "Config.h"
#import "SINJWT.h"
#import <OSLog/OSLog.h>

// These functions are defined in Sinch SDK
extern void sin_client_set_support_active_connection(id<SINClient>, BOOL);
extern void sin_client_start_active_connection(id<SINClient>);

@interface SINClientMediator () <SINClientDelegate>
@property (nonatomic, readwrite, strong) _Nullable id<SINClient> client;
@property (nonatomic, readwrite, strong) os_log_t customLog;
@property (nonatomic, readwrite, strong) _Nullable ErrorCompletion clientCreatedCallback;
@end

@implementation SINClientMediator

- (instancetype)init {
  self = [super init];

  if (self != nil) {
    self.customLog = os_log_create("com.sinch.sdk.app", "SINClientMediator");
  }

  return self;
}

- (void)createClientWithUserId:(NSString *)userId completion:(ErrorCompletion)completion {
  if (self.client == nil) {
    self.clientCreatedCallback = completion;
    NSError *error;
    self.client = [Sinch clientWithApplicationKey:APPLICATION_KEY
                                  environmentHost:ENVIRONMENT_HOST
                                           userId:userId
                                            error:&error];

    if (!self.client && completion) {
      completion(error);
      return;
    }
    self.client.delegate = self;

    sin_client_set_support_active_connection(_client, YES);
    [self.client start];

    // WARNING: Do NOT use "Active Connection" in a production application.
    // Use Push Notifications instead. (see SINManagedPush and -[SINClient enableManagedPushNotifications])
    sin_client_start_active_connection(self.client);
  } else {
    if (completion && self.client.isStarted) {
      completion(nil);
    }
  }
}

- (void)logout:(VoidCompletion)completion {
  if (self.client != nil) {
    if (self.client.isStarted) {
      // Remove push registration from Sinch backend
      [self.client unregisterPushNotificationDeviceToken];
      [self.client terminateGracefully];
    }

    self.client = nil;
  }

  if (completion != nil) {
    completion();
  }
}

#pragma mark - SINClientDelegate

- (void)client:(id<SINClient>)client requiresRegistrationCredentials:(id<SINClientRegistration>)registrationCallback {
  // WARNING: Embedding the Application Secret and use of the class SINJWT as shown here
  // should NOT be done for an application deployed to production.
  // The class SINJWT is only here to show how to construct and sign a registration token.

  NSString *jwt = [SINJWT jwtForUserRegistrationWithApplicationKey:APPLICATION_KEY
                                                 applicationSecret:APPLICATION_SECRET
                                                            userId:_client.userId];

  [registrationCallback registerWithJWT:jwt];
}

- (void)clientDidStart:(id<SINClient>)client {
  os_log(self.customLog, "Sinch client started successfully (version: %{public}@)", [Sinch versionString]);

  if (self.clientCreatedCallback != nil) {
    self.clientCreatedCallback(nil);
    self.clientCreatedCallback = nil;
  }
}

- (void)clientDidFail:(id<SINClient>)client error:(NSError *)error {
  os_log_error(self.customLog, "Sinch client error: %{public}@", [error localizedDescription]);

  self.client = nil;

  if (self.clientCreatedCallback != nil) {
    self.clientCreatedCallback(error);
    self.clientCreatedCallback = nil;
  }
}

@end
