#import "CallViewController.h"
#import "CallViewController+UI.h"
#import <Sinch/SINUIView+Fullscreen.h>
#import "SINClientMediator.h"

// Usage of gesture recognizers to toggle between front- and back-camera and fullscreen.
//
// * Triple tap the local preview view to pause sending local video stream to remote.
// * Double tap the local preview view to switch between front- and back-camera.
// * Single tap the local video stream view to make it go full screen.
//     Tap again to go back to normal size.
// * Single tap the remote video stream view to make it go full screen.
//     Tap again to go back to normal size.

@interface CallViewController () <SINCallDelegate>
@property (nonatomic, readonly) id<SINAudioController> audioController;
@property (nonatomic, readonly) id<SINVideoController> videoController;
@property (nonatomic, weak) IBOutlet UIGestureRecognizer *remoteVideoFullscreenGestureRecognizer;
@property (nonatomic, weak) IBOutlet UIGestureRecognizer *localVideoFullscreenGestureRecognizer;
@property (nonatomic, weak) IBOutlet UIGestureRecognizer *switchCameraGestureRecognizer;
@property (nonatomic, weak) IBOutlet UITapGestureRecognizer *pauseResumeVideoRecognizer;
@property (nonatomic, readwrite) BOOL videoPaused;
@end

@implementation CallViewController

- (id<SINAudioController>)audioController {
  return self.clientMediator.client.audioController;
}

- (id<SINVideoController>)videoController {
  return self.clientMediator.client.videoController;
}

- (void)setCall:(id<SINCall>)call {
  _call = call;
  _call.delegate = self;
}

#pragma mark - UIViewController Cycle

- (void)viewDidLoad {
  [super viewDidLoad];

  NSAssert(self.clientMediator != nil, @"self.clientMediator must not be nil here", @"");
  NSAssert(self.clientMediator.client != nil, @"self.clientMediator.client must not be nil here", @"");

  if ([self.call direction] == SINCallDirectionIncoming) {
    [self setCallStatusText:@""];
    [self showButtons:kButtonsAnswerDecline];
    [[self audioController] startPlayingSoundFile:[self pathForSound:@"incoming.wav"] loop:YES error:nil];
  } else {
    [self setCallStatusText:@"calling..."];
    [self showButtons:kButtonsHangup];
  }

  if ([self.call.details isVideoOffered]) {
    [self.localVideoView addSubview:[[self videoController] localView]];
    [self.localVideoFullscreenGestureRecognizer requireGestureRecognizerToFail:self.switchCameraGestureRecognizer];
    [self.localVideoFullscreenGestureRecognizer requireGestureRecognizerToFail:self.pauseResumeVideoRecognizer];
    [self.switchCameraGestureRecognizer requireGestureRecognizerToFail:self.pauseResumeVideoRecognizer];
    [[[self videoController] localView] addGestureRecognizer:self.localVideoFullscreenGestureRecognizer];
    [[[self videoController] localView] addGestureRecognizer:self.pauseResumeVideoRecognizer];
    [[[self videoController] localView] addGestureRecognizer:self.switchCameraGestureRecognizer];
    [[[self videoController] remoteView] addGestureRecognizer:self.remoteVideoFullscreenGestureRecognizer];
    _videoPaused = NO;
  }
}

- (void)viewWillAppear:(BOOL)animated {
  [super viewWillAppear:animated];
  self.remoteUsername.text = [self.call remoteUserId];
  // [[self audioController] enableSpeaker];
}

#pragma mark - Call Actions

- (IBAction)accept:(id)sender {
  [[self audioController] stopPlayingSoundFile];
  [self.call answer];
}

- (IBAction)decline:(id)sender {
  [self.call hangup];
  [self dismiss];
}

- (IBAction)hangup:(id)sender {
  [self.call hangup];
  [self dismiss];
}

- (IBAction)onSwitchCameraTapped:(id)sender {
  AVCaptureDevicePosition current = self.videoController.captureDevicePosition;
  self.videoController.captureDevicePosition = SINToggleCaptureDevicePosition(current);
}

- (IBAction)onFullScreenTapped:(id)sender {
  UIView *view = [sender view];
  if ([view sin_isFullscreen]) {
    view.contentMode = UIViewContentModeScaleAspectFit;
    [view sin_disableFullscreen:YES];
  } else {
    view.contentMode = UIViewContentModeScaleAspectFill;
    [view sin_enableFullscreen:YES];
  }
}

- (IBAction)onPauseResumeVideoTapped:(id)sender {
  if (_videoPaused) {
    [self.call resumeVideo];
    _videoPaused = NO;
  } else {
    [self.call pauseVideo];
    _videoPaused = YES;
  }
}

- (void)onDurationTimer:(NSTimer *)unused {
  NSInteger duration = (NSInteger)[[NSDate date] timeIntervalSinceDate:[[self.call details] establishedTime]];
  [self setDuration:duration];
}

#pragma mark - SINCallDelegate

- (void)callDidProgress:(id<SINCall>)call {
  [self setCallStatusText:@"ringing..."];
  [[self audioController] startPlayingSoundFile:[self pathForSound:@"ringback.wav"] loop:YES error:nil];
}

- (void)callDidEstablish:(id<SINCall>)call {
  [self startCallDurationTimerWithSelector:@selector(onDurationTimer:)];
  self.remoteVideoView.hidden = NO;
  [self showButtons:kButtonsHangup];
  [[self audioController] stopPlayingSoundFile];
}

- (void)callDidEnd:(id<SINCall>)call {
  [self dismiss];
  [[self audioController] stopPlayingSoundFile];
  [self stopCallDurationTimer];
  [[[self videoController] remoteView] removeFromSuperview];
  [[self audioController] disableSpeaker];
}

- (void)callDidAddVideoTrack:(id<SINCall>)call {
  [self.remoteVideoView addSubview:[[self videoController] remoteView]];
  // The callee can answer the call before the video track is available, or after. In makes sense to show
  // remoteVideoView as soon as it's available in the first case only
  self.remoteVideoView.hidden = (call.state != SINCallStateEstablished);
}

- (void)callDidPauseVideoTrack:(id<SINCall>)call {
  self.remoteVideoView.hidden = YES;
}

- (void)callDidResumeVideoTrack:(id<SINCall>)call {
  self.remoteVideoView.hidden = NO;
}

#pragma mark - Sounds

- (NSString *)pathForSound:(NSString *)soundName {
  return [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:soundName];
}

@end
